(function () {
  if (window.AEJSBridge) {
    return;
  }
  var eventCallbacks = {};
  var dispatchCallbacks = {};
  var CUSTOM_PROTOCOL_SCHEME = 'yy';
  var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';
  var uniqueId = 1;

  /**
   * 事件注册
   * @param req.handlerName 事件名称
   * @param req.callback 回调函数
   */
  function addEventListener(req) {
    if (!req || !req.handlerName || !req.callback) {
      return;
    }
    if (!eventCallbacks[req.handlerName]) {
      eventCallbacks[req.handlerName] = [];
    }
    eventCallbacks[req.handlerName].push(req.callback);
    dispatch({handlerName:req.handlerName});
  }

  /**
   * 取消事件注册
   * @param req.handlerName 事件名称
   * @param req.execFunc 事件名称
   * @param req.callback 回调函数
   */
  function removeEventListener(req) {
    if (!req || !req.handlerName || !req.callback) {
      return;
    }
    var callArray = eventCallbacks[req.handlerName];

    var delSuccess = false;
    for (var index = 0; index < callArray.length; index++) {
      if (callArray[index] === req.callback) {
        callArray.splice(index, 1);
        delSuccess = true;
        break;
      }
    }
//    req.callback({
//      msg: delSuccess ? 'delete success ~' : 'delete failed ~',
//      code: delSuccess ? 0 : -1
//    });
  }

  /**
   * 事件分发
   * @param req.handlerName 事件类型 String
   * @param req.params 事件数据 (String | JsonString 可以为null)
   * @param req.callback 回调函数 (function)
   */
  function dispatch(req) {
    if (!req || !req.handlerName) {
      return;
    }
    if (req.params == undefined) {
        req.params = {};
    }
    if (req.callback) {
      var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
      dispatchCallbacks[callbackId] = req.callback;
      req.callbackId = callbackId;
    }
    var l = [];
    l.push(req);
    toonJSBridge.setJsCall(CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(JSON.stringify(l)));
  }

    function _combineMultipartData(msg) {
      var responseId = msg.responseId;
      var no = msg.no;
      var total = msg.total;

      var multipartData = AEJSBridge.multipartDataQueues[responseId];
      if (!multipartData) {
          multipartData = new Array();
          AEJSBridge.multipartDataQueues[responseId] = multipartData;
      }
      multipartData[no] = msg.data;

      if (no == total) {
        var fullData = multipartData.join('');
        delete AEJSBridge.multipartDataQueues[responseId];
        return [true, fullData];
      }

      return [false, ''];
    }

  /**
   * 提供给native使用
   * @param messageJSON
   * {
   *   handlerName: '',
   *   callbackId: '',
   *   responseId: '',
   *   responseData: '',
   *   params: '',
   *   msg: '',
   *   code: ''
   * }
   * @private
   */
  function _dispatchMessageFromNative(messageJSON) {
    setTimeout(function () {
      var msg = JSON.parse(messageJSON);
      // completed: 是否合并完成，用于区分分包过程中返回的空和合并后也是空的情况
      // fullData: 合并data的分包数据，如果合并完成，返回完整的data，否者返回''，继续下次合并
      var [completed, fullData] = _combineMultipartData(msg);
      if (!completed) {
          return;
      }

      var data;
      var responseCallback;
      try {
          data = JSON.parse(base64Decode(fullData));
      } catch(e){
          data = base64Decode(fullData);
        //console.log(e);   // native向H5传字符串时会导致调试工具报json错误，暂时先注掉此行，后期处理
      }

      // 数字和bool类型的data，JSON.parse会转换类型，改为使用原始的msg.data数据
      if (typeof data == 'number' || typeof data == 'boolean') {
          data = base64Decode(fullData);
      }

//      alert(JSON.stringify(data));
      // Js dispatch callback
      if (msg.responseId) {
        responseCallback = dispatchCallbacks[msg.responseId];
        if (!responseCallback) {
          return;
        }
        responseCallback({
          data: data,
          msg: msg.msg,
          code: msg.code
        });
        delete dispatchCallbacks[msg.responseId];
        // Js addEventListener callback
      } else {
        var callbacks;
        if (!msg.handlerName) {
          console.log("handlerName is empty ...");
          return;
        }
        callbacks = eventCallbacks[msg.handlerName];
        if (!callbacks) {
          console.log("handlerName callbacks is empty ...");
          return;
        }
        callbacks.forEach(function (item) {
          item({
            data: data,
            msg: msg.msg,
            code: msg.code
          });
        })
      }
    });
  }

   function base64Decode(str) {
       return decodeURIComponent(atob(str).split('').map(function(c) {
           return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
       }).join(''));
    }
  // 提供给native调用,receiveMsgs 在会在页面加载完后赋值为null
  function _handleMessageFromNative(messageJSON) {
    _dispatchMessageFromNative(messageJSON);
  }


  var AEJSBridge = window.AEJSBridge = {
    addEventListener: addEventListener,
    removeEventListener: removeEventListener,
    dispatch: dispatch,
    multipartDataQueues: [],
    _handleMessageFromNative: _handleMessageFromNative
  };

  var doc = document;
  var readyEvent = doc.createEvent('Events');
  readyEvent.initEvent('AEJSBridgeReady');
  readyEvent.bridge = AEJSBridge;
  doc.dispatchEvent(readyEvent);
})();
